/*
   C-  uni_graph_viewer.pp,    X-  Imlib2
*/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <Imlib2.h>
#include <stdio.h>
#include <stdlib.h>
#include <libgen.h>
 

#define DATA08 unsigned char
DATA32 *gl_d_32;
DATA08 *gl_d_08;

#define PIXR(x, y)	gl_d_08[(((y) * gl_w + (x)) << 2) + 2]
#define PIXG(x, y)	gl_d_08[(((y) * gl_w + (x)) << 2) + 1]
#define PIXB(x, y)	gl_d_08[(((y) * gl_w + (x)) << 2) + 0]
#define PIX_(x, y, l)	gl_d_08[(((y) * gl_w + (x)) << 2) + l]

#define PIX(x, y)	PIXR(x, y) = PIXG(x, y) = PIXB(x, y)
int gl_w, gl_h;


//   
Display		*disp;
Visual		*visual;
Colormap	cm;
Window		win;

int		nScreenNum;
Screen		*prScreen;

int		Ready = 0;
int		MaxW, MaxH;
int		PicRW, PicRH;
int		WinW, WinH;
int		diser = 0;
int		FontW, FontH;

Imlib_Font	font;
Imlib_Image	image, commentTop, commentBut;

#define  ikcEscape	0
#define  ikcUp		1
#define  ikcDn		2
#define  ikcLf		3
#define  ikcRg		4
#define  ikcReturn	5
#define  ikcR		6


void to_right_write(char *msg) {
  int w, h;
  imlib_get_text_size(msg, &w, &h);
  imlib_text_draw(WinW - w, 0, msg);
}


void to_center_write(char *msg) {
  int w, h;
  imlib_get_text_size(msg, &w, &h);
  imlib_text_draw((WinW - w) / 2, 0, msg);
}


void make_image(unsigned char *scr, int w, int h, int prefer_scale_w, int prefer_scale_h, char *comment, char **key_comments) {
/*
        ,  
     X  Y
       ,     X:Y = HardProp
*/
//  printf("  %d*%d x %d*%d\n\r", w, prefer_scale_w, h, prefer_scale_h);
  int LocalMaxW = MaxW - 100, LocalMaxH = MaxH - 100;

  if ( (w * prefer_scale_w) < LocalMaxW && (h * prefer_scale_h) < LocalMaxH ) {
    // ,  ޣ ,    
    //     
    int sc;
    for (sc = 1; (w * prefer_scale_w * sc) < LocalMaxW && (h * prefer_scale_h * sc) < LocalMaxH; sc++) ;
    prefer_scale_w *= sc - 1; PicRW = w * prefer_scale_w;
    prefer_scale_h *= sc - 1; PicRH = h * prefer_scale_h;
//    printf("    : %d x %d\n\r", prefer_scale_w, prefer_scale_h);
  } else {
    //   .     Y,     
    //       X     Y
    int Scale;
    Scale = LocalMaxW / w;
    PicRW = Scale *  w;
    PicRH = Scale * (h * prefer_scale_h) / prefer_scale_w;

//    printf("   .     Y,    X = %d\n\r", Scale);
    if (PicRH > LocalMaxH) {
      //      Y   X,       
      Scale = LocalMaxH / h;
      PicRH = Scale *  h;
      PicRW = Scale * (w * prefer_scale_w) / prefer_scale_h;

//      printf("  .     X,    Y = %d\n\r", Scale);
      if (PicRW > LocalMaxW) { printf("-  -   :E\n\r"); exit(33); };
    };
  };

  WinW = PicRW; WinH = FontH * 2 + PicRH;
  XMoveResizeWindow(disp, win, (MaxW - WinW) / 2, (MaxH - WinH) / 2, WinW, WinH);

/*
    ̣    imlib
*/
  image = imlib_create_image(w, h);
  imlib_context_set_image(image);
  gl_d_32 = imlib_image_get_data();

  int x, y;
  int pal[16] = {
    0x000000, 0x7f0000, 0x007f00, 0x7f7f00, 0x00007f, 0x7f007f, 0x007f7f, 0x7f7f7f,
    0x404040, 0xff0000, 0x00ff00, 0xffff00, 0x0000ff, 0xff00ff, 0x00ffff, 0xffffff
  };
  for (x = 0; x < w; x++) for (y = 0; y < h; y++) {
    gl_d_32[y * w + x] = pal[ scr[x * 512 + y] & 0xF ];
  };

/*
       
*/
  commentTop = imlib_create_image(WinW, FontH);
  imlib_context_set_image(commentTop);
  imlib_context_set_color(0, 0, 100, 255);
  imlib_image_fill_rectangle(0, 0, WinW, FontH);
  imlib_context_set_color(255, 255, 255, 255);

  if ( key_comments[ikcLf] ) imlib_text_draw(0, 0, key_comments[ikcLf]);
  if ( key_comments[ikcUp] ) to_center_write( key_comments[ikcUp] );
  if ( key_comments[ikcRg] ) to_right_write( key_comments[ikcRg] );


  commentBut = imlib_create_image(WinW, FontH);
  imlib_context_set_image(commentBut);
  imlib_context_set_color(0, 0, 100, 255);
  imlib_image_fill_rectangle(0, 0, WinW, FontH);
  imlib_context_set_color(255, 255, 255, 255);

  imlib_text_draw(0, 0, "Space - change dither");
  if ( key_comments[ikcDn] ) to_center_write( key_comments[ikcDn] );
  to_right_write("F2 - save image");
}


void drop_image() {
  imlib_context_set_image(commentTop);
  imlib_free_image();

  imlib_context_set_image(image);
  imlib_free_image();

  imlib_context_set_image(commentBut);
  imlib_free_image();
}


void x11_Init(int argc, char **argv) {
  disp = XOpenDisplay(NULL);
  if (! disp) return;
  imlib_context_set_display(disp);

  nScreenNum = XDefaultScreen(disp);
  prScreen = XScreenOfDisplay(disp, nScreenNum);

  visual = XDefaultVisual(disp, nScreenNum);
  imlib_context_set_visual(visual);

  cm = XDefaultColormap(disp, nScreenNum);
  imlib_context_set_colormap(cm);

//  imlib_add_path_to_font_path("/ms-dos/MustDie:/WINDOWS/Fonts/");
//  imlib_add_path_to_font_path("/usr/home/deka/.gimp-2.6/fonts");
  imlib_add_path_to_font_path(dirname(argv[0]));
  if (font = imlib_load_font("GARA/15")) {
    imlib_context_set_font(font);
    imlib_get_text_size("H", &FontW, &FontH);
  } else {
    printf("!!    !! [%s]\n\r", dirname(argv[0]));
    exit(252);
  };

  win = XCreateSimpleWindow(
    disp, DefaultRootWindow(disp),
    0, 0, 10, 10,		10,
    XWhitePixel(disp, nScreenNum), XBlackPixel(disp, nScreenNum)
  );

  XSelectInput(disp, win, StructureNotifyMask | ExposureMask | KeyPressMask);
  imlib_context_set_drawable(win);

  Ready = 1;
}


void set_window_name(char *wndname) {
  XTextProperty tp;
  char **pwndname = &wndname;

  XStringListToTextProperty(pwndname, 1, &tp);
  XSetWMName(disp, win, &tp);
}


void save_image(char *name) {
  char FName[1024], *i;

  imlib_context_set_image(image);
  imlib_image_set_format("tif");
  snprintf(FName, sizeof(FName), "%s.TIF", name);
  for (i = FName; *i != 0; i++) if (*i == '/') *i = '_';
  imlib_save_image(FName);
}


void expose() {
  imlib_context_set_anti_alias(1);
  imlib_context_set_dither(1);

  imlib_context_set_image(commentTop);
  imlib_render_image_on_drawable_at_size(0,             0,  WinW, FontH);

  imlib_context_set_image(commentBut);
  imlib_render_image_on_drawable_at_size(0, FontH + PicRH,  WinW, FontH);

  imlib_context_set_image(image);
  imlib_context_set_anti_alias(diser);
  imlib_context_set_dither(diser);
  imlib_render_image_on_drawable_at_size(0,         FontH, PicRW, PicRH);
}


int x11_Open() {
  if (Ready) {
    XMapWindow(disp, win);
    MaxW = XWidthOfScreen(prScreen); MaxH = XHeightOfScreen(prScreen);

    XEvent ev;
    do {			//     ,     x11_Open  x11_Close
      XNextEvent(disp, &ev);	//     XUnmapWindow
    } while (ev.type != MapNotify);
  };
  return Ready;
}


int x11_Expose(void *p, int w, int h, int psw, int psh, char *comment, char **key_comments) {
  XEvent	ev;
  KeySym	KeyInResult = 0;

  make_image(p, w, h, psw, psh, comment, key_comments);
  set_window_name(comment);
  expose();

  while (1) {
    XNextEvent(disp, &ev);
    switch (ev.type) {

      case Expose:
	if (ev.xexpose.count == 0) expose();
	break;

      case ConfigureNotify:
        WinW = ev.xconfigure.width;
        WinH = ev.xconfigure.height;
	PicRW = WinW;
	PicRH = WinH - FontH * 2;
        break;

      case KeyPress:
	KeyInResult = XKeycodeToKeysym(disp, ev.xkey.keycode, 0 /* KeyInState */);
	switch (KeyInResult) {
	  case XK_Left	: if ( key_comments[ikcLf] ) { drop_image(); return ikcLf; }; break;
	  case XK_Right	: if ( key_comments[ikcRg] ) { drop_image(); return ikcRg; }; break;
	  case XK_Up	: if ( key_comments[ikcUp] ) { drop_image(); return ikcUp; }; break;
	  case XK_Down	: if ( key_comments[ikcDn] ) { drop_image(); return ikcDn; }; break;
	  case XK_Return: if ( key_comments[ikcReturn] ) { drop_image(); return ikcReturn; }; break;
	  case XK_Escape:
	  case XK_F3	: if ( key_comments[ikcEscape] ) { drop_image(); return ikcEscape; }; break;
	  case XK_F12	:
	  case XK_F2	: save_image(comment); break;
	  case 32	: diser ^= 1; expose(); break;
	  default:
	    printf("KeySym = %X\n\r", KeyInResult);
	};
	break;

    }
  }
}


void x11_Close() {
  if (Ready) {
    XUnmapWindow(disp, win);
    XFlush(disp);
  };
}
