{
        graph_proc    Graph
      BMP
}
unit uni_graph_viewer;

interface

uses host_os_specific_p, world_formats, char_sets, subwindows, graph, mywrite, crt_keys;


//   /  
//  True   
function  GraphOpen: Boolean;

//     w * h   readonly-  512x512 
//     -  Σ,  4-  ,  
// prefer_scale_?      
// comment    ,       
// ikc -    ,  ...
//     (, ),  "" 
//    ,    != Nil
function  GraphExpose(const p: Pointer; const w, h, prefer_scale_w, prefer_scale_h: LongInt; const FName: tAgatName; const comment: PChar; const ikc: tIntKeyComments): tIntKeyCode;

//  
procedure GraphClose;


implementation


{$PackRecords 1}
type
  tBMPFileHead	=	Record
    bfType	: Array[0..1] of char;	//  'BM')
    bfSize	: Longint;		//    dword  
    bfReserved	: Longint;
    bfOffbits	: Longint;		//        
  end;

  tBMPInfoHead	=	Record
    biSize		: Longint; {   }
    biWidth		: Longint; {      }
    biHeight		: Longint; {      }
    biPlanes		: Word;	  {     }
    biBitCount		: Word;	  {     }
    biCompression	: Longint; {   }
    biSizeImage		: Longint; {     }
    biXPelsPerMeter	: Longint; {   , / }
    biYPelsPerMeter	: Longint; {   , / }
    biClrUsed		: Longint; {    }
    biClrImportant	: Longint; {  ""  }
  end;

  tBMPPalette		= Array[0..15] of Record
    B, G, R, res	: Byte;
  end;


const
  Palette		: tBMPPalette = (
    (B: $00; G: $00; R: $00; res: 0),
    (B: $00; G: $00; R: $7f; res: 0),
    (B: $00; G: $7f; R: $00; res: 0),
    (B: $00; G: $7f; R: $7f; res: 0),

    (B: $7f; G: $00; R: $00; res: 0),
    (B: $7f; G: $00; R: $7f; res: 0),
    (B: $7f; G: $7f; R: $00; res: 0),
    (B: $7f; G: $7f; R: $7f; res: 0),

    (B: $40; G: $40; R: $40; res: 0),
    (B: $00; G: $00; R: $ff; res: 0),
    (B: $00; G: $ff; R: $00; res: 0),
    (B: $00; G: $ff; R: $ff; res: 0),

    (B: $ff; G: $00; R: $00; res: 0),
    (B: $ff; G: $00; R: $ff; res: 0),
    (B: $ff; G: $ff; R: $00; res: 0),
    (B: $ff; G: $ff; R: $ff; res: 0)
  );


//     BMP-
function uniq_name: String;
const n: LongInt = 0; BaseName = 'image';
var f: File; s: String; i: Byte;
begin
  FileMode := 2;
  repeat
    str(n: 2, s);
    for i := 1 to Length(s) do if s[i] = ' ' then s[i] := '0';
    s := BaseName + '-' + s + '.bmp'; uniq_name := s;
    {$I-}
    assign(f, s); reset(f);
    if IOResult = 0 then begin
    {$I+}
      close(f); inc(n);
    end else Exit;
  until False;
end;


//    16-  BMP 
procedure SaveBMP(const fn: String; const x0, y0, lx, ly: LongInt);
const BitCount = 4; PalSize = 16;
var
  f		: File;
  FileHead	: tBMPFileHead;
  InfoHead	: tBMPInfoHead;
  i		: Byte;
  x, y, lx_byte	: LongInt;
  buf_pix	: Array[0..639] of Byte;

begin
  lx_byte := (lx+7) div 2 and $fffffffc;

  FileMode := 1;
  Assign(f, fn); ReWrite(f, 1);

  with FileHead do begin
    bfType		:= 'BM';
    bfSize		:= (lx_byte * ly) + (4*PalSize) + 54;
    bfReserved		:= 0;
    bfOffbits		:= (4 * PalSize) + 54;
  end;

  with InfoHead do begin
    biSize		:= SizeOf(InfoHead);
    biWidth		:= lx;
    biHeight		:= ly;
    biPlanes		:= 1;
    biBitCount		:= BitCount;
    biCompression	:= 0;
    biSizeImage		:= ly * lx;
    biXPelsPerMeter	:= 0;
    biYPelsPerMeter	:= 0;
    biClrUsed		:= 0;
    biClrImportant	:= 0;
  end;

  BlockWrite(f, FileHead, SizeOf(FileHead));
  BlockWrite(f, InfoHead, SizeOf(InfoHead));
  BlockWrite(f, Palette, PalSize*4);

  for y := ly-1 downto 0 do begin
    for x := 0 to lx_byte-1 do buf_pix[x] := GetPixel(x*2 + x0, y + y0) shl 4 or GetPixel(x*2 + 1 + x0, y + y0);
    BlockWrite(f, buf_pix, lx_byte);
  end;

  Close(f);
end;



function  GraphOpen: Boolean;
const
  pal: Array[0..15] of Cardinal = (
    $000000, $7f0000, $007f00, $7f7f00, $00007f, $7f007f, $007f7f, $7f7f7f,
    $404040, $ff0000, $00ff00, $ffff00, $0000ff, $ff00ff, $00ffff, $ffffff
  );

var gd, gm: Integer; i: LongInt;
begin
  gd := D8Bit; gm := m1024x768; InitGraph(gd, gm, '');
  GraphOpen := GraphResult = grOk;

  for i := 0 to 15 do begin
    SetPalette(i, i);
    SetRGBPalette(i,
      (pal[i] shr 16) and $FF,
      (pal[i] shr  8) and $FF,
      (pal[i]       ) and $FF
    );
  end;
  SetColor(15);
end;

procedure GraphClose;
begin
  CloseGraph;
end;


const
  MaxW		= 1024;		//      
  MaxH		= 768 - 20;	//     8x8
  LocalMaxW	= MaxW;		// -       ,
  LocalMaxH	= MaxH;		//       
  				//     

var
  PicX0, PicY0	: LongInt;	//       ޣ 
  PicRW, PicRH	: LongInt;	//   ,    
  ScaleX, ScaleY: LongInt;	//  ;     prefer_scale,
  				//     ޣ ,  ...

procedure calc_scale(const w, h, prefer_scale_w, prefer_scale_h: LongInt);
var sc: LongInt;
begin
{
        ,  
     X  Y
       ,     X:Y = HardProp
}
  if (w * prefer_scale_w < LocalMaxW) and (h * prefer_scale_h < LocalMaxH ) then begin

    // ,  ޣ ,    
    //     
    sc := 1;
    while (w * prefer_scale_w * sc < LocalMaxW) and (h * prefer_scale_h * sc < LocalMaxH) do inc(sc);
    ScaleX := (sc - 1) * prefer_scale_w;
    ScaleY := (sc - 1) * prefer_scale_h;
    PicRW := w * ScaleX;
    PicRH := h * ScaleY;

  end else begin

    //   .   ,      
    //       X  ,    , Y
    ScaleX := LocalMaxW div w;
    PicRW := ScaleX * w;
    ScaleY := Round(ScaleX * prefer_scale_h / prefer_scale_w);
    PicRH := ScaleY * h;

    if PicRH > LocalMaxH then begin

      //      Y   X,       
      ScaleY := LocalMaxH div h;
      PicRH := ScaleY * h;
      ScaleX := Round(ScaleY * prefer_scale_w / prefer_scale_h);
      PicRW := ScaleX * w;

      if PicRW > LocalMaxW then begin
        outtextxy(1, 1, 'Hren s matematikoy');
	readln;
	halt(33);
      end;

    end;

  end;
  PicX0 := (LocalMaxW - PicRW) div 2;
  PicY0 := (LocalMaxH - PicRH) div 2 + 10; //    
end;


procedure text_out(const ha, va: ShortInt; const msg: PChar);
var x, y, w: LongInt;
begin
  if msg = Nil then Exit;
  w := Length(msg) * 8;
  case ha of
    -1: x := 0;
     0: x := (GetMaxX - w) div 2;
    +1: x := GetMaxX - w;
  end;
  case va of
    -1: y := 0;
    +1: y := GetMaxY - 8;
  end;
  outtextxy(x, y, msg);
end;


function  GraphExpose(const p: Pointer; const w, h, prefer_scale_w, prefer_scale_h: LongInt; const FName: tAgatName; const comment: PChar; const ikc: tIntKeyComments): tIntKeyCode;
type
  tScr		= Array[0..511, 0..511] of Byte;

var
  x, y, x_s, y_s: LongInt;
  scr		: ^tScr;

begin
  if (w = 0) or (h = 0) then Exit( ikcEscape );
  calc_scale(w, h, prefer_scale_w, prefer_scale_h);
  ClearDevice;

  scr := p;
  for x := 0 to w - 1 do begin
    x_s := x * ScaleX + PicX0;
    for y := 0 to h - 1 do begin
      y_s := y * ScaleY + PicY0;
      SetFillStyle(1, scr^[x, y] and $F);
      bar(x_s, y_s, x_s + ScaleX - 1, y_s + ScaleY - 1);
    end;
  end;

  text_out(-1, -1, ikc[ikcLf]);
  text_out( 0, -1, ikc[ikcUp]);
  text_out(+1, -1, ikc[ikcRg]);

  text_out(-1, +1, comment);
  text_out( 0, +1, ikc[ikcDn]);
  text_out(+1, +1, 'F2 - save image');

  repeat
    case mReadKey of

      HKEY_Left		: if ikc[ikcLf]		<> Nil then Exit(ikcLf);
      HKEY_Right	: if ikc[ikcRg]		<> Nil then Exit(ikcRg);
      HKEY_Up		: if ikc[ikcUp]		<> Nil then Exit(ikcUp);
      HKEY_Down		: if ikc[ikcDn]		<> Nil then Exit(ikcDn);
      13		: if ikc[ikcReturn]	<> Nil then Exit(ikcReturn);
      27,
      HKEY_F3		: if ikc[ikcEscape]	<> Nil then Exit(ikcEscape);
      HKEY_F12,
      HKEY_F2		: SaveBMP(uniq_name, PicX0, PicY0, PicRW, PicRH);

    end;
  until False;
end;



end.
